-- main for headlights.love

function love.load()
	gr, li, pi			= love.graphics, love.image, math.pi
	gr.setFont( gr.newFont( 18 ) )
	gr.setCaption( 'Head Lights' )
	beam = {
		width			= pi / 8,
		reach			= 200,
		separation		= 50,
		intensity		= 1,
		}
	backgnd_intensity	= 0.3
	image				= gr.newImage( 'flower.jpg' )
	width, height 		= image:getWidth(), image:getHeight()
	angle				= 0
	----------------
	-- create canvas
	----------------
	mask_canvas			= gr.newCanvas( width, height )
	load_effect()
	load_mask()
	info = {
			" Keyboard Controls",
			"<q><w> beam reach",
			"<a><s>  beam width",
				"<z><x>  beam separation",
			"<n><m> beam intensity",
			"<j><k>   background intensity"
			}
	end

function love.draw()
	gr.setPixelEffect( effect )
	gr.draw( image )
	gr.setPixelEffect()
	gr.print('FPS: '..love.timer.getFPS(),gr.getWidth()-100,10)
	key_options()
	show_data()
	end

function love.update(dt)
	angle = angle + dt
	load_mask()
	end

function love.keypressed(key)
	if key == 'escape'	then love.event.push('quit') end
	if key == 'q'		then beam.reach			= beam.reach - 10		end
	if key == 'w'		then beam.reach			= beam.reach + 10		end
	if key == 'a'		then beam.width			= beam.width - pi/32	end
	if key == 's'		then beam.width			= beam.width + pi/32	end
	if key == 'z'		then beam.separation	= beam.separation - 10	end
	if key == 'x'		then beam.separation	= beam.separation + 10	end
	if key == 'j'		then backgnd_intensity	= backgnd_intensity > 0	and backgnd_intensity - .1 or 0	end
	if key == 'k'		then backgnd_intensity	= backgnd_intensity < 1 and backgnd_intensity + .1 or 1	end
	if key == 'n' 		then beam.intensity		= beam.intensity > 0	and beam.intensity - 0.1 or 0	end
	if key == 'm'		then beam.intensity		= beam.intensity < 1	and beam.intensity + 0.1 or 1	end
	end

function draw_on_canvas()
	gr.setBackgroundColor( 0, 0, 0, 255 )	-- black background
	gr.setColor( 255, 255, 255, 255 )		-- white hole in mask
	local radx, rady	= 250, 150				-- path radius
	local x 			= radx * math.sin( -angle ) + width / 2
	local y 			= rady * math.cos( -angle ) + height / 2
	local x2 			= (radx+beam.separation) * math.sin( -angle ) + width / 2
	local y2 			= (rady+beam.separation) * math.cos( -angle ) + height / 2
	local angle1, angle2= angle-beam.width+pi, angle + beam.width+pi
	gr.arc( 'fill', x, y, beam.reach, angle1, angle2, 100 )
	gr.arc( 'fill',x2,y2, beam.reach, angle1, angle2, 100 )
	end

function load_effect()
	effect = gr.newPixelEffect [[
	extern Image  mask;
	extern number intsBG;
	extern number intsFG;
	vec4 effect(vec4 color,Image tex,vec2 tc,vec2 pc)
	{	vec4 img_color		= Texel( tex, tc );
		vec4 mask_color		= Texel( mask, tc );
		number isWhite		= (mask_color.r + mask_color.b + mask_color.b);
		if ( isWhite    	> 1 )
			{
			img_color.a = intsFG; // show image if mask is white
			return img_color;}
			else
			{img_color.a = intsBG; // mask image outside beam
			return img_color;
			}
		}
	]]
	end

function load_mask()
	mask_canvas:clear()
	--------------------
	-- draw on canvas --
	--------------------
	mask_canvas:renderTo( draw_on_canvas )
	---------------------------
	-- update effect setings --
	---------------------------
	effect:send( 'mask', mask_canvas )
	effect:send( 'intsBG', backgnd_intensity )
	effect:send( 'intsFG', beam.intensity)
	end

function key_options()
	gr.setColor( 155, 155, 155, 255 )
	local x, y = 10, 20
	for i = 1, #info do
		gr.print( info[i], x, i * y )
		end
	end

function show_data()
	data = {
		"Settings",
		"beam.reach: "..beam.reach,
		"beam.width: "..2*math.floor(beam.width*180/pi),
		"beam.separation: "..beam.separation,
		"beam.alpha: "..beam.intensity,
		"backgnd alpha: "..backgnd_intensity
		}
	local x, y = 10, 20
	for i = 1, #data do
		gr.print( data[i], x, i * y+ 440 )
		end
	end

